
import { push, each as _each, merge, lowercase, isDefined, isFunction, isString, makeArray, isWindow, isUndefined, isObject, NODE_TYPE_DOCUMENT, NODE_TYPE_ATTRIBUTE, NODE_TYPE_TEXT, NODE_TYPE_COMMENT, NODE_TYPE_DOCUMENT_FRAGMENT, NODE_TYPE_ELEMENT, extend } from './selector/utils';
import domReady from 'bplokjs-dom-utils/domReady';
import parseHTML from './selector/parseHTML';
import _find from './selector/find';
import _offset from 'bplokjs-dom-utils/offset';
import _position from './position';
import _offsetParent from 'bplokjs-dom-utils/offsetParent';
import matches from 'bplokjs-dom-utils/matches';
import { hasClass as _hasClass, addClass as _addClass, removeClass as _removeClass, toggleClass as _toggleClass } from 'bplokjs-dom-utils/classes';
import _closest from 'bplokjs-dom-utils/closest';
import _css from 'bplokjs-dom-utils/css';
var rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;
var BOOLEAN_ATTR = {};

_each('multiple,selected,checked,disabled,readOnly,required,open'.split(','), function (i, value) {
  BOOLEAN_ATTR[lowercase(value)] = value;
}); //////////////////////////////////////////////


export default function JQLite(selector, context) {
  if (!(this instanceof JQLite)) {
    return new JQLite(selector, context);
  }

  if (selector instanceof JQLite) {
    return selector;
  } // HANDLE: $(""), $(null), $(undefined), $(false)


  if (!selector) {
    return this;
  }

  var match, elem;

  if (typeof selector === "string") {
    if (selector[0] === "<" && selector[selector.length - 1] === ">" && selector.length >= 3) {
      // Assume that strings that start and end with <> are HTML and skip the regex check
      match = [null, selector, null];
    } else {
      match = rquickExpr.exec(selector);
    } // Match html or make sure no context is specified for #id


    if (match && (match[1] || !context)) {
      // HANDLE: $(html) -> $(array)
      if (match[1]) {
        context = context instanceof JQLite ? context[0] : context; // Option to run scripts is true for back-compat
        // Intentionally let the error be thrown if parseHTML is not present

        merge(this, parseHTML(match[1], context && context.nodeType ? context.ownerDocument || context : document, true));
        return this; // HANDLE: $(#id)
      } else {
        elem = document.getElementById(match[2]);

        if (elem) {
          // Inject the element directly into the jQuery object
          this[0] = elem;
          this.length = 1;
        }

        return this;
      } // HANDLE: $(expr, $(...))

    } else if (!context) {
      return _find(selector, document, new JQLite()); // HANDLE: $(expr, context)
      // (which is just equivalent to: $(context).find(expr)
    } else {
      return JQLite(context).find(selector);
    } // HANDLE: $(DOMElement)

  } else if (selector.nodeType) {
    this[0] = selector;
    this.length = 1;
    return this; // HANDLE: $(function)
    // Shortcut for document ready
  } else if (isFunction(selector)) {
    domReady(selector);
  }

  return makeArray(selector, this);
}
var JQLitePrototype = JQLite.fn = JQLite.prototype = {
  ready: domReady,
  toString: function toString() {
    var value = [];

    _each(this, function (i, e) {
      value.push('' + e);
    });

    return '[' + value.join(', ') + ']';
  },
  eq: function eq(index) {
    return index >= 0 ? JQLite(this[index]) : JQLite(this[this.length + index]);
  },
  length: 0,
  push: push,
  sort: [].sort,
  splice: [].splice,
  extend: extend,
  find: function find(selector) {
    var i,
        ret,
        len = this.length,
        self = this;
    ret = new JQLite();

    for (i = 0; i < len; i++) {
      _find(selector, self[i], ret);
    }

    return ret; //len > 1 ? jQuery.uniqueSort( ret ) : ret;	
  },
  each: function each(callback) {
    return _each(this, callback);
  }
};

if (typeof Symbol === "function") {
  var arr = [];
  JQLitePrototype[Symbol.iterator] = arr[Symbol.iterator];
}

function jqLiteWidthOrHeightCreator(type) {
  var funcName = lowercase(type);
  return function (elem, value) {
    if (isWindow(elem)) {
      return elem.document.documentElement["client" + type];
    }

    if (elem.nodeType === NODE_TYPE_DOCUMENT) {
      var doc = elem.documentElement; // Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
      // whichever is greatest

      return Math.max(elem.body["scroll" + type], doc["scroll" + type], elem.body["offset" + type], doc["offset" + type], doc["client" + type]);
    }

    var exp1 = type === 'Width' ? 'Left' : 'Top';
    var exp2 = type === 'Width' ? 'Right' : 'Bottom'; //getter

    if (value === undefined) {
      return elem.offsetWidth - parseFloat(_css(elem, "border" + exp1 + type)) - parseFloat(_css(elem, "padding" + exp1)) - parseFloat(_css(elem, "padding" + exp2)) - parseFloat(_css(elem, "border" + exp2 + type)); //setter		
    } else {
      var isBorderBox = _css(elem, "boxSizing") === "border-box";
      var borderOrPadding = parseFloat(_css(elem, "border" + exp1 + type)) + parseFloat(_css(elem, "padding" + exp1)) + parseFloat(_css(elem, "padding" + exp2)) + parseFloat(_css(elem, "border" + exp2 + type));

      _css(elem, funcName, !isBorderBox || value === "" ? value : (value || 0) - borderOrPadding);
    }
  };
}

function jqLiteAttr(element, name, value) {
  var ret;
  var nodeType = element.nodeType;

  if (nodeType === NODE_TYPE_TEXT || nodeType === NODE_TYPE_ATTRIBUTE || nodeType === NODE_TYPE_COMMENT || !element.getAttribute) {
    return;
  }

  var lowercasedName = lowercase(name);
  var isBooleanAttr = BOOLEAN_ATTR[lowercasedName];

  if (isDefined(value)) {
    // setter
    if (value === null || value === false && isBooleanAttr) {
      element.removeAttribute(name);
    } else {
      element.setAttribute(name, isBooleanAttr ? lowercasedName : value);
    }
  } else {
    // getter
    ret = element.getAttribute(name);

    if (isBooleanAttr && ret !== null) {
      ret = lowercasedName;
    } // Normalize non-existing attributes to undefined (as jQuery).


    return ret === null ? undefined : ret;
  }
}

function jqLiteProp(element, name, value) {
  if (isDefined(value)) {
    element[name] = value;
  } else {
    return element[name];
  }
} ////////////Methods////////////////


_each({
  append: function append(node) {
    node = new JQLite(node);
    return this.each(function (i, elem) {
      var nodeType = elem.nodeType;
      if (nodeType !== NODE_TYPE_ELEMENT && nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT) return;

      for (var _i = 0, ii = node.length; _i < ii; _i++) {
        var child = node[_i];
        elem.appendChild(child);
      }
    });
  },
  prepend: function prepend(node) {
    node = new JQLite(node);
    return this.each(function (i, elem) {
      if (elem.nodeType === NODE_TYPE_ELEMENT) {
        var index = elem.firstChild;

        _each(node, function (i, child) {
          elem.insertBefore(child, index);
        });
      }
    });
  },
  remove: function remove() {
    return this.each(function (elem) {
      var parent = elem.parentNode;
      if (parent) parent.removeChild(elem);
    });
  },
  children: function children() {
    var children = [];
    this.each(function (i, elem) {
      _each(elem.childNodes, function (ii, element) {
        if (element.nodeType === NODE_TYPE_ELEMENT) {
          children.push(element);
        }
      });
    });
    return children;
  },
  width: function width(value) {
    var length = this.length;
    if (!length) return value === undefined ? undefined : this;
    var func = jqLiteWidthOrHeightCreator('Width');

    if (value === undefined) {
      var elem = this[0];
      return func(elem);
    } else {
      for (var i = 0; i < length; i++) {
        func(this[i], value);
      }
    }

    return this;
  },
  height: function height(value) {
    var length = this.length;
    if (!length) return value === undefined ? undefined : this;
    var func = jqLiteWidthOrHeightCreator('Height');

    if (value === undefined) {
      var elem = this[0];
      return func(elem);
    } else {
      for (var i = 0; i < length; i++) {
        func(this[i], value);
      }
    }

    return this;
  },
  outerWidth: function outerWidth() {
    var length = this.length;
    if (!length) return;
    var elem = this[0];

    if (isWindow(elem)) {
      return elem.innerWidth;
    }

    if (elem.nodeType === NODE_TYPE_DOCUMENT) return this.width();
    return elem.offsetWidth;
  },
  outerHeight: function outerHeight() {
    var length = this.length;
    if (!length) return;
    var elem = this[0];

    if (isWindow(elem)) {
      return elem.innerHeight;
    }

    if (elem.nodeType === NODE_TYPE_DOCUMENT) return this.height();
    return elem.offsetHeight;
  },
  offset: function offset(coordinates) {
    if (coordinates) {
      return this.each(function (i, elem) {
        _offset(elem, coordinates);
      });
    }

    if (!this.length) return;
    return _offset(this[0]);
  },
  position: function position() {
    if (!this.length) return;
    return _position(this[0]);
  },
  css: function css(key, value) {
    if (value !== undefined) {
      return this.each(function (i, elem) {
        _css(elem, key, value);
      });
    } else if (isObject(key)) {
      for (var k in key) {
        this.css(k, key[k]);
      }

      return this;
    } else if (this.length) {
      return _css(this[0], key);
    }
  },
  attr: function attr(key, value) {
    if (value !== undefined) {
      return this.each(function (i, elem) {
        jqLiteAttr(elem, key, value);
      });
    } else if (isObject(key)) {
      for (var k in key) {
        this.attr(k, key[k]);
      }

      return this;
    } else if (this.length) {
      return jqLiteAttr(this[0], key);
    }
  },
  removeAttr: function removeAttr(name) {
    return this.each(function (i, elem) {
      elem.removeAttribute(name);
    });
  },
  prop: function prop(key, value) {
    if (value !== undefined) {
      return this.each(function (i, elem) {
        jqLiteProp(elem, key, value);
      });
    } else if (this.length) {
      return jqLiteProp(this[0], key, value);
    }
  },
  filter: function filter(selector) {
    var elems = new JQLite(),
        elem,
        i,
        len;

    if (isFunction(selector)) {
      for (i = 0, len = this.length; i < len; i++) {
        elem = this[i];

        if (selector.call(elem, i, elem)) {
          elems.push(elem);
        }
      }
    } else if (isString(selector)) {
      for (i = 0, len = this.length, elem; i < len; i++) {
        elem = this[i];

        if (matches(elem, selector)) {
          elems.push(elem);
        }
      }
    }

    return elems;
  },
  closest: function closest(selector) {
    var nodes = [];
    this.each(function (i, elem) {
      var node = _closest(elem, selector);

      if (node) nodes.push(node);
    });
    return new JQLite(nodes);
  },
  offsetParent: function offsetParent() {
    if (this.length) return _offsetParent(this[0]);
  },
  hasClass: function hasClass(className) {
    for (var i = 0, n = this.length; i < n; i++) {
      if (_hasClass(this[i], className)) {
        return true;
      }
    }

    return false;
  },
  addClass: function addClass(className) {
    if (!className) return this;
    return this.each(function (i, elem) {
      className.split(/\s+/).forEach(function (name) {
        return _addClass(elem, name);
      });
    });
  },
  removeClass: function removeClass(className) {
    if (!className) return this;
    return this.each(function (i, elem) {
      className.split(/\s+/).forEach(function (name) {
        return _removeClass(elem, name);
      });
    });
  },
  toggleClass: function toggleClass(className) {
    if (!className) return this;
    return this.each(function (i, elem) {
      className.split(/\s+/).forEach(function (name) {
        return _toggleClass(elem, name);
      });
    });
  },
  html: function html(value) {
    if (isUndefined(value)) {
      if (this.length) return this[0].innerHTML;
    } else {
      return this.each(function (i, elem) {
        elem.innerHTML = value;
      });
    }
  },
  empty: function empty() {
    return this.each(function (i, elem) {
      while (elem.firstChild) {
        elem.removeChild(elem.firstChild);
      }
    });
  }
}, function (name, func) {
  /**
      * Properties: writes return selection, reads return first value
      */
  JQLitePrototype[name] = func;
});

_each({
  scrollLeft: "pageXOffset",
  scrollTop: "pageYOffset"
}, function (method, prop) {
  var top = "pageYOffset" === prop;

  JQLitePrototype[method] = function (val) {
    //getter
    if (val === undefined) {
      var win;
      var elem = this[0];
      if (!elem) return;

      if (isWindow(elem)) {
        win = elem;
      } else if (elem.nodeType === NODE_TYPE_DOCUMENT) {
        win = elem.defaultView;
      }

      return win ? win[prop] : elem[method];
    } //setter


    return this.each(function (i, elem) {
      var win;

      if (isWindow(elem)) {
        win = elem;
      } else if (elem.nodeType === NODE_TYPE_DOCUMENT) {
        win = elem.defaultView;
      }

      if (win) {
        win.scrollTo(!top ? val : win.pageXOffset, top ? val : win.pageYOffset);
      } else {
        elem[method] = val;
      }
    });
  };
}); ////////////Functions////////////////


_each({
  extend: extend,
  each: _each,
  isWindow: isWindow,
  css: _css
}, function (name, func) {
  JQLite[name] = func;
});